import React from 'react';
// import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { InputText } from 'primereact/inputtext';
import moment from 'moment';
import PropTypes from 'prop-types';
import SimpleReactValidator from '../validator';
import InputCalendarComponent from '../components/inputs/InputCalendarComponent';
import ActionButton from '../components/ActionButton';
import Dropdown from '../components/base/Dropdown';
import CustomDataTable from '../components/base/CustomDataTable';

class BaseListWithEditContainer extends React.Component {
	constructor(props, service) {
		super(props);
		this.service = service;
		this.handlePage = this.handlePage.bind(this);
		this.refreshTable = this.refreshTable.bind(this);
		this.actionTemplate = this.actionTemplate.bind(this);
		this.handleChange = this.handleChange.bind(this);
		this.addItemButton = this.addItemButton.bind(this);
		this.inputTextEditor = this.inputTextEditor.bind(this);
		this.dropdownEditor = this.dropdownEditor.bind(this);
		this.calendarEditor = this.calendarEditor.bind(this);
		this.requiredValidator = this.requiredValidator.bind(this);
		this.updateTableElement = this.updateTableElement.bind(this);
		this.validator = new SimpleReactValidator();
		this._isMounted = false;
	}

	componentDidMount() {
		this._isMounted = true;
		this.init();
	}

	componentWillUnmount() {
		this._isMounted = false;
	}

	init() {
		this.setState(
			{
				loading: true,
			},
			() => this.refreshTable()
		);
	}

	updateTableElement() {}

	handleChange(inputType, parameter, event, onAfterStateChange) {
		const { addElement } = this.state;
		let varName;
		let varValue;
		if (event !== undefined) {
			switch (inputType) {
				case 'MULTI_DROPDOWN':
					varName = event.target.name;
					varValue = event.target.value ? event.target.value : undefined;
					if (this._isMounted) {
						const modifiedList = addElement[varName];
						modifiedList[parameter] = varValue;
						this.setState(
							prevState => ({
								addElement: {
									...prevState.addElement,
									[varName]: modifiedList,
								},
							}),
							() => (onAfterStateChange ? onAfterStateChange() : null)
						);
					}
					break;
				case 'DROPDOWN':
					varName = event.target.name;
					varValue = event.target.value ? event.target.value : undefined;
					const varValueEnum = varValue ? varValue[parameter] : undefined;
					if (this._isMounted) {
						if (parameter) {
							this.setState(
								prevState => ({
									addElement: {
										...prevState.addElement,
										[`${varName}Obj`]: varValue,
										[varName]: varValueEnum,
									},
								}),
								() => (onAfterStateChange ? onAfterStateChange() : null)
							);
						} else {
							this.setState(
								prevState => ({
									addElement: {
										...prevState.addElement,
										[varName]: varValue,
									},
								}),
								() => (onAfterStateChange ? onAfterStateChange() : null)
							);
						}
					}
					break;
				case 'CHECKBOX':
					varName = event.target.name;
					varValue = event.checked ? event.checked : false;
					if (this._isMounted) {
						this.setState(
							prevState => ({
								addElement: {
									...prevState.addElement,
									[varName]: varValue,
								},
							}),
							() => (onAfterStateChange ? onAfterStateChange() : null)
						);
					}
					break;
				case 'MULTI_SELECT':
				case 'CALENDAR_FROM':
				case 'AUTOCOMPLETE':
				case 'CALENDAR':
				case 'TEXTAREA':
				case 'TEXT':
				default:
					varName = event.target.name;
					varValue =
						event.target.value || event.target.value === '' ? event.target.value : undefined;
					if (this._isMounted) {
						this.setState(
							prevState => ({
								addElement: {
									...prevState.addElement,
									[varName]: varValue,
								},
							}),
							() => (onAfterStateChange ? onAfterStateChange() : null)
						);
					}
					break;
			}
		} else {
			console.log('handleChange implementation error');
		}
	}

	refreshTable() {
		const { modifiedList } = this.props;
		const { criteria, objectId } = this.state;
		if (objectId !== null && objectId !== undefined) {
			this.service.getEditList(criteria).then(data => {
				if (data.content !== undefined) {
					data.content.forEach(element => {
						this.updateTableElement(element);
						if (
							modifiedList.get(element.id) !== null &&
							modifiedList.get(element.id) !== undefined
						) {
							this.updateElement(element, modifiedList.get(element.id));
						}
					});
				}
				this.updateList(data.content, data.totalElements);
			});
		} else {
			this.updateList([], 0);
		}
	}

	updateList(contentList, totalElements) {
		const { addedList, modifiedList } = this.props;
		const { criteria } = this.state;
		const page = criteria.page + 1;
		const limit = criteria.limit;
		if (contentList === undefined) {
			contentList = [];
		}
		if (page * limit > totalElements) {
			const addedPage = Math.ceil((page * limit - totalElements) / limit);
			if (page * limit - totalElements > limit) {
				contentList = contentList.concat(
					addedList.slice((page - addedPage) * limit - totalElements, limit)
				);
			} else {
				contentList = contentList.concat(addedList.slice(0, page * limit - totalElements));
			}
		}
		this.setState({
			loading: false,
			list: contentList,
			size: totalElements + modifiedList.size + addedList.length,
		});
	}

	onSort(event) {
		this.setState(
			prevState => ({
				loading: true,
				criteria: {
					...prevState.criteria,
					sortField: event.sortField,
					sortDirection: event.sortOrder > 0 ? 'ASC' : 'DESC',
				},
			}),
			() => this.refreshTable()
		);
	}

	handlePage(event) {
		this.setState(
			prevState => ({
				loading: true,
				first: event.first,
				criteria: {
					...prevState.criteria,
					page: event.page,
					limit: event.rows,
				},
			}),
			() => this.refreshTable()
		);
	}

	onEditorValueChange(props, value) {
		const { handleEditElement } = this.props;
		const updatedList = [...props.value];
		updatedList[props.rowIndex][props.field] = value;
		this.setState({ list: updatedList });
		handleEditElement(props.rowData, null);
	}

	inputTextEditor(props) {
		return (
			<InputText
				key={`${props.field}-edit`}
				type="text"
				value={props.rowData[props.field]}
				onChange={e => this.onEditorValueChange(props, e.target.value)}
			/>
		);
	}

	calendarEditor(props) {
		const { viewMode } = this.props;
		return (
			<InputCalendarComponent
				id={props.field}
				name={props.field}
				label={props.header}
				value={props.rowData[props.field]}
				onChange={(a, b, e) => this.onEditorValueChange(props, e.target.value)}
				dateFormat="dd-mm-yy"
				placeholder="dd-mm-rrrr"
				viewMode={viewMode}
				showLabel={false}
				showIcon
				insideTable
				validator={this.validator}
				validators={props.validators ? props.validators : 'required|date'}
			/>
		);
	}

	dropdownEditor(props, items) {
		return (
			<Dropdown
				key={`${props.field}-edit`}
				value={props.rowData[props.field]}
				options={items}
				onChange={e => this.onEditorValueChange(props, e.value)}
				style={{ width: '100%' }}
				placeholder="Wybierz"
			/>
		);
	}

	requiredValidator(props) {
		const value = props.rowData[props.field];
		if (value instanceof Date) {
			return moment(value).isValid();
		} else {
			return value && value.length > 0;
		}
	}

	actionTemplate(rowData) {
		const { handleEditElement, viewMode } = this.props;
		if (viewMode !== 'VIEW') {
			if (rowData.deleted) {
				return (
					<ActionButton
						label={'Cofnij'}
						handleClick={e => {
							e.preventDefault();
							handleEditElement(rowData, false);
						}}
						key={'edit-button'}
					/>
				);
			} else {
				return (
					<ActionButton
						label={'Usuń'}
						handleClick={e => {
							e.preventDefault();
							handleEditElement(rowData, true);
						}}
						key={'delete-button'}
					/>
				);
			}
		}
	}

	addItemButton() {
		return (
			<ActionButton
				label={'Dodaj wartość'}
				handleClick={e => {
					e.preventDefault();
					this.setState({ addDialogVisible: true });
				}}
				key={'add-button'}
			/>
		);
	}

	isUserInRole(role) {
		const { roles } = this.props;
		const rolesArray = roles.split(',');
		return rolesArray.includes(role);
	}

	isUserInAnyRole(...rolesToFind) {
		const { roles } = this.props;
		const rolesArray = roles.split(',');
		rolesToFind.forEach(role => {
			if (rolesArray.includes(role)) {
				return true;
			}
		});
		return false;
	}

	renderDataTable(columns) {
		const { editable } = this.props;
		const { criteria, first, list, loading, size } = this.state;
		const dynamicColumns = columns.map(col => {
			return (
				<Column
					key={col.key ? col.key : col.field}
					field={col.field}
					header={col.header}
					body={col.body}
					className={col.className}
					sortable={col.sortable}
					editor={col.editor}
					editorValidator={col.validator}
				/>
			);
		});
		return (
			<CustomDataTable
				key="data-table"
				emptyMessage="Brak rekordów do wyświetlenia"
				responsive
				value={list}
				paginator
				rows={criteria.limit}
				editable={editable}
				totalRecords={size}
				lazy
				first={first}
				onPage={this.handlePage}
				rowsPerPageOptions={[5, 10, 20, 50, 100]}
				onSort={e => this.onSort(e)}
				sortField={criteria.sortField}
				sortOrder={criteria.sortDirection === 'ASC' ? 1 : -1}
				loading={loading}
			>
				{dynamicColumns}
			</CustomDataTable>
		);
	}
}

BaseListWithEditContainer.defaultProps = {
	editable: false,
	viewMode: 'VIEW',
	roles: '',
};

BaseListWithEditContainer.propTypes = {
	addedList: PropTypes.array.isRequired,
	editable: PropTypes.bool,
	handleEditElement: PropTypes.func.isRequired,
	modifiedList: PropTypes.array.isRequired,
	roles: PropTypes.string,
	viewMode: PropTypes.string,
};

export default BaseListWithEditContainer;
