/* eslint-disable no-undef */
/* eslint-disable no-unused-vars */
/* eslint-disable prefer-const */
/* eslint-disable react/jsx-handler-names */
import React from 'react';
import PropTypes from 'prop-types';
import Tooltip from 'primereact/tooltip';
import classNames from 'classnames';

export class Checkbox extends React.Component {
	constructor(props) {
		super(props);
		this.state = {};

		this.onClick = this.onClick.bind(this);
		this.onFocus = this.onFocus.bind(this);
		this.onBlur = this.onBlur.bind(this);
		this.onKeyDown = this.onKeyDown.bind(this);
	}

	componentDidMount() {
		const { tooltip } = this.props;
		if (tooltip) {
			this.renderTooltip();
		}
	}

	componentDidUpdate(prevProps) {
		const { checked, tooltip } = this.props;
		this.input.checked = checked;

		if (prevProps.tooltip !== tooltip) {
			if (this.tooltip) this.tooltip.updateContent(tooltip);
			else this.renderTooltip();
		}
	}

	componentWillUnmount() {
		if (this.tooltip) {
			this.tooltip.destroy();
			this.tooltip = null;
		}
	}

	onClick(e) {
		const { checked, id, name, disabled, onChange, readOnly, tooltip, value } = this.props;
		if (!disabled && !readOnly && onChange) {
			onChange({
				originalEvent: e,
				value,
				checked: !checked,
				stopPropagation: () => {},
				preventDefault: () => {},
				target: {
					type: 'checkbox',
					name,
					id,
					value,
					checked: !checked,
				},
			});

			this.input.checked = !checked;
			this.input.focus();

			e.preventDefault();
		}
	}

	onFocus() {
		this.setState({ focused: true });
	}

	onBlur() {
		this.setState({ focused: false });
	}

	onKeyDown(event) {
		if (event.key === 'Enter') {
			this.onClick(event);
			event.preventDefault();
		}
	}

	renderTooltip() {
		const { tooltip, tooltipOptions } = this.props;
		this.tooltip = new Tooltip({
			target: this.element,
			content: tooltip,
			options: tooltipOptions,
		});
	}

	render() {
		const {
			ariaDescribedBy,
			ariaLabel,
			ariaLabelledBy,
			checked,
			className,
			disabled,
			id,
			inputId,
			name,
			onContextMenu,
			onKeyDown,
			onMouseDown,
			readOnly,
			required,
			style,
		} = this.props;
		const { focused } = this.state;
		let containerClass = classNames('p-checkbox p-component', className);
		let boxClass = classNames('p-checkbox-box p-component', {
			'p-highlight': checked,
			'p-disabled': disabled,
			'p-focus': focused,
		});
		let iconClass = classNames('p-checkbox-icon p-c', { 'pi pi-check': checked });

		return (
			<div
				ref={el => (this.element = el)}
				id={id}
				className={containerClass}
				style={style}
				onClick={this.onClick}
				onContextMenu={onContextMenu}
				onMouseDown={onMouseDown}
			>
				<div className="p-hidden-accessible">
					<input
						type="checkbox"
						aria-label={ariaLabel}
						aria-labelledby={ariaLabelledBy}
						aria-describedy={ariaDescribedBy}
						aria-required={required}
						ref={el => (this.input = el)}
						id={inputId}
						name={name}
						defaultChecked={checked}
						onKeyDown={onKeyDown ? onKeyDown : this.onKeyDown}
						onFocus={this.onFocus}
						onBlur={this.onBlur}
						disabled={disabled}
						readOnly={readOnly}
						required={required}
					/>
				</div>
				<div
					className={boxClass}
					ref={el => (this.box = el)}
					role="checkbox"
					aria-checked={checked}
				>
					<span className={iconClass} />
				</div>
			</div>
		);
	}
}

Checkbox.defaultProps = {
	id: null,
	inputId: null,
	value: null,
	name: null,
	checked: false,
	style: null,
	className: null,
	disabled: false,
	required: false,
	readOnly: false,
	tooltip: null,
	tooltipOptions: null,
	ariaLabelledBy: null,
	onChange: null,
	onMouseDown: null,
	onContextMenu: null,
};

Checkbox.propTypes = {
	ariaDescribedBy: PropTypes.string,
	ariaLabel: PropTypes.string,
	ariaLabelledBy: PropTypes.string,
	checked: PropTypes.bool,
	className: PropTypes.string,
	disabled: PropTypes.bool,
	id: PropTypes.string,
	inputId: PropTypes.string,
	name: PropTypes.string,
	onChange: PropTypes.func,
	onContextMenu: PropTypes.func,
	onKeyDown: PropTypes.func,
	onMouseDown: PropTypes.func,
	readOnly: PropTypes.bool,
	required: PropTypes.bool,
	style: PropTypes.object,
	tooltip: PropTypes.string,
	tooltipOptions: PropTypes.object,
	value: PropTypes.any,
};

export default Checkbox;
